//loggers.java Summer 2003
//Features:
// log levels (file and console)
// sync to disk
// log chat

import java.io.*;
//import java.util.Date;

public class loggers extends Thread
{
	static final int version = 7;

	static PrintWriter chat, err;
	static int syncseconds = 240;
	static String fileLevel;
	static String consoleLevel;
	static boolean      bOpen = false;
	static boolean bKeepGoing = false;

	public loggers(config cfg)
	{
		String chatname  = cfg.GetStr("log_chat", "LogFile");
		String errname   = cfg.GetStr("Log", "LogFile");
			
		syncseconds = cfg.GetInt("Persist", "SyncSeconds", 300);
		fileLevel   = cfg.GetStr("log_file", "all");
		if (fileLevel == null) fileLevel = "x";

		consoleLevel = cfg.GetStr("log_console", "all");
		if (consoleLevel == null) consoleLevel = "x";

		try
		{
			if (chatname != null)
			{
				chat = new PrintWriter(
					new FileOutputStream(chatname, true)
					);
			} else chat = null;
			if (errname != null)
			{
				err = new PrintWriter(
					new FileOutputStream(errname, true)
					);
			} else err = null;

			bOpen = true;
			bKeepGoing = true;
			this.start();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		String time = (new java.util.Date()).toString();
		Log("I", "<loggers> Log file open: " + time);
		Log("  Log file open: " + chatname);
	}

	public void OpenChatLog(String name)
	{
		if (chat != null)
		{
			Log("E", "<loggers> Tried to open chat log when " +
				"already open");
			chat.close();
		}
	
		try
		{
			chat = new PrintWriter(
				new FileOutputStream(name, true)
				);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public void CloseChatLog()
	{
		if (chat != null) chat.close();
		chat = null;
	}

	public void Log(String text)
	{
		if (!bOpen) return;
		if (chat == null) return;
		chat.println(text);
	}

	public void Log(String level, String text)
	{
		if (consoleLevel.indexOf(level) >= 0)
			System.err.println(level + " " + text);
		
		if (!bOpen) return;
		if (err == null) return;
		if (fileLevel.indexOf(level) >= 0)
			err.println(level + " " + text);
	}

	public void run()
	{
		while(bKeepGoing)
		{
			try
			{
				Thread.sleep( syncseconds*1000 );
			}
			catch(InterruptedException e)
			{
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			if (bOpen)
			{
				Log("D", "<loggers> Syncing to disk");
				if (chat != null) chat.flush();
				if (err != null) err.flush();
			}
		}
	}

	public void close()
	{
		if (!bOpen) return;
		CloseChatLog();
		if (err != null) err.close();
		bOpen = false;
		bKeepGoing = false;
		this.interrupt();
	}

}
